@@ -31,12 +31,12 @@ import Data.Text.Encoding (decodeUtf8', encodeUtf8)
 import Debug.Trace
 import Skylighting.Regex
 import Skylighting.Types
+import Skylighting.Parser (resolveKeywords)
 import Data.List.NonEmpty (NonEmpty((:|)), (<|), toList)
 #if !MIN_VERSION_base(4,11,0)
 import Data.Semigroup
 #endif
 
-
 newtype ContextStack = ContextStack{ unContextStack :: NonEmpty Context }
   deriving (Show)
 
@@ -134,13 +134,17 @@ instance MonadError String TokenizerM where
 tokenize :: TokenizerConfig -> Syntax -> Text -> Either String [SourceLine]
 tokenize config syntax inp =
   eitherStack >>= \stack ->
-    case runTokenizerM action config (startingState stack) of
+    case runTokenizerM action
+            config{ syntaxMap = Map.map (resolveKeywords (syntaxMap config))
+                                          (syntaxMap config) }
+            (startingState stack) of
        (_, Success ls) -> Right ls
        (_, Error e)    -> Left e
        (_, Failure)    -> Left "Could not tokenize code"
   where
     action = mapM tokenizeLine (zip (BS.lines (encodeUtf8 inp)) [1..])
-    eitherStack = case lookupContext (sStartingContext syntax) syntax of
+    eitherStack = case lookupContext (sStartingContext syntax)
+                         (resolveKeywords (syntaxMap config) syntax) of
                     Just c  -> Right $ ContextStack (c :| [])
                     Nothing -> Left "No starting context specified"
     startingState stack =
@@ -291,7 +295,10 @@ tryRule rule inp = do
                 HlCStringChar -> withAttr attr $ parseCStringChar inp
                 HlCChar -> withAttr attr $ parseCChar inp
                 Float -> withAttr attr $ parseFloat inp
-                Keyword kwattr kws -> withAttr attr $ keyword kwattr kws inp
+                Keyword _kwattr (Left listname) ->
+                  throwError $ "Keyword with unresolved list " <> show listname
+                Keyword kwattr (Right kws) ->
+                  withAttr attr $ keyword kwattr kws inp
                 StringDetect s -> withAttr attr $
                                     stringDetect (rDynamic rule) (rCaseSensitive rule)
                                                  s inp
@@ -527,7 +534,7 @@ regExpr dynamic re inp = do
         Just (matchedBytes, capts) -> do
           unless (null capts) $
              modify $ \st -> st{ captures =
-                                  IntMap.map (toSlice matchedBytes) capts }
+                                  IntMap.map (toSlice inp) capts }
           takeChars (UTF8.length matchedBytes)
         _ -> mzero
 
@@ -575,7 +582,7 @@ getCapture :: Int -> TokenizerM Text
 getCapture capnum = do
   capts <- gets captures
   case IntMap.lookup capnum capts of
-     Nothing -> mzero
+     Nothing -> throwError $ "Capture " <> show capnum <> " not defined!"
      Just x  -> decodeBS x
 
 keyword :: KeywordAttr -> WordSet Text -> ByteString -> TokenizerM Text
