@@ -15,6 +15,7 @@ module Skylighting.Types (
               , WordSet(..)
               , makeWordSet
               , inWordSet
+              , ListItem(..)
               , Matcher(..)
               , Rule(..)
               , Context(..)
@@ -104,7 +105,8 @@ data Matcher =
   | StringDetect !Text
   | WordDetect !Text
   | RegExpr !RE
-  | Keyword KeywordAttr !(WordSet Text)
+  | Keyword !KeywordAttr (Either Text (WordSet Text))
+      -- Either Left list name (unresolved) or Right wordset (resolved)
   | Int
   | Float
   | HlCOct
@@ -143,12 +145,20 @@ data Rule = Rule{
 
 instance Binary Rule
 
+-- | A list item is either just a textual value or an included list.
+-- IncludeList (x,y) includes list y from syntax with full name x.
+data ListItem = Item !Text | IncludeList !(Text, Text)
+  deriving (Show, Eq, Ord, Read, Data, Typeable, Generic)
+
+instance Binary ListItem
+
 -- | A syntax corresponds to a complete Kate syntax description.
 -- The 'sShortname' field is derived from the filename.
 data Syntax = Syntax{
     sName            :: !Text
   , sFilename        :: !String
   , sShortname       :: !Text
+  , sLists           :: !(Map.Map Text [ListItem])
   , sContexts        :: !(Map.Map Text Context)
   , sAuthor          :: !Text
   , sVersion         :: !Text
@@ -290,7 +300,7 @@ defStyle = TokenStyle {
   , tokenUnderline  = False
   }
 
--- | A color (red/green/blue).
+-- | A color (red, green, blue).
 data Color = RGB Word8 Word8 Word8
   deriving (Show, Read, Ord, Eq, Data, Typeable, Generic)
 
@@ -617,7 +627,7 @@ ansi256ColorList = [ (Xterm256ColorCode 232, RGB 8 8 8) -- grayscale colors
 instance ToColor Xterm256ColorCode where
     toColor = flip lookup ansi256ColorList -- cannot actually fail
 
--- | JSON @"#1aff2b" corresponds to the color @RGB 0x1a 0xff 0x2b@.
+-- | JSON @"#1aff2b"@ corresponds to the color @RGB 0x1a 0xff 0x2b@.
 instance FromJSON Color where
   parseJSON (String t) = maybe mempty return $ toColor (Text.unpack t)
   parseJSON _          = mempty
